function [c ceq] = dyn_nonl_con(z,wn,routing,ti,m0,m_ss,...
    general_parameters,well_parameters,constr)
% Nonlinear constraint function based on simulation 

% Input variables:
% z - decision variables, inputs for considered wells for each time
%       interval
%       z = [z1; z2; z3; ... zn;]
% wn - well numbers in consideration, e.g. [1 2 4 5 7]
% routing - routing of the wells to flowline 0 or 1
%       e.g. [0 1 1 0 0]
% ti - time intervals for inputs in minutes, last element is endtime
%       e.g. [0 60 120 180]
%       minimum start and end time
% m0 - initial states for simulation, [3x7] matrix
% m_ss - steady-state solution at end, [3x(length(nw))] matrix
% constr - system constraints [gas liquid water gaslift]


rho_o = general_parameters(7);
rho_w = general_parameters(8);

par = well_parameters(:,wn);
dt = 20; % Simulation time steps, seconds
nw = length(wn);

% Simulate
iter = 1;
niter = 60*ti(end)/dt+1;
t = 0;
in = 1;
next_int = 60*ti(in+1);
u = z(2*(in-1)*nw+1:2*in*nw);
mm = zeros(3,nw,niter);
mm(:,:,iter) = m0(:,wn);
yy = zeros(11,nw,niter);
while t < 60*ti(end)
    iter = iter+1;
    
    % Simulation, one time step
    for w = 1:nw;
        well = par(:,w);
        mw = mm(:,w,iter-1);
        uw = u(2*w-1:2*w);
        [mdot meas] = well_model(mw,uw,well);
        yy(:,w,iter) = meas;
        mm(:,w,iter) = mm(:,w,iter-1) + mdot*dt;
    end
    t = t + dt;

    % Update time interval and inputs
    if t >= next_int
        in = in + 1;
        if in < length(ti)
            next_int = 60*ti(in+1);
            u = z(2*(in-1)*nw+1:2*in*nw);
        end
    end
end


% Overall production constraints

% Get measurements and wc parameter
w_gp = []; w_gp(:,:) = yy(6,:,:);
w_lp = []; w_lp(:,:) = yy(5,:,:);
r_wc = par(4,:);
% Initialize
gp0 = zeros(1,niter);
lp0 = gp0;
wp0 = gp0;
gp1 = gp0;
lp1 = gp0;
wp1 = gp0;
% Calculate production from each well
for w = 1:nw
    % Gas in Sm3/day
    gpw = w_gp(w,:)/0.8247*86400;
    % Liquid in bpd
    lpw = (r_wc(w)*w_lp(w,:)/rho_w + (1-r_wc(w))*w_lp(w,:)/rho_o)...
        /0.158987295*86400;
    % Water in bpd
    wpw = r_wc(w)*w_lp(w,:)/rho_w/0.158987295*86400;
    % Summarize for each flowline
    if routing(w)
        gp1 = gp1 + gpw;
        lp1 = lp1 + lpw;
        wp1 = wp1 + wpw;
    else
        gp0 = gp0 + gpw;
        lp0 = lp0 + lpw;
        wp0 = wp0 + wpw;
    end
end
% Set constraints for max and mean production
tol = 10; % [Percent] tolerance for max production
cp = [...
    max(gp0) - (1+tol/100)*constr(1);
    max(lp0) - (1+tol/100)*constr(2);
    max(wp0) - (1+tol/100)*constr(3);
    max(gp1) - (1+tol/100)*constr(1);
    max(lp1) - (1+tol/100)*constr(2);
    max(wp1) - (1+tol/100)*constr(3);
    mean(gp0) - constr(1);
    mean(lp0) - constr(2);
    mean(wp0) - constr(3);
    mean(gp1) - constr(1);
    mean(lp1) - constr(2);
    mean(wp1) - constr(3);
    ];
c = 1e-9*cp;

% c = [];


% End constraint, m_end = m_ss
ceq = [];
for i = 1:nw
    ss = m_ss(3*i-2:3*i)';
    m_end = [];
    m_end(:,:) = mm(:,i,niter);
    ceq = [ceq; 1e-3*(m_end - ss)./ss];
end









    
    
    
    
