function m = steady_state(u,well)
    % Function to calculate steady-state masses for well with inputs u, and
    % parameters given in well

    global general_parameters;
    
    tol = 1e-7;
    m0_t = [500 10000]';
    m0_a = 1000;
    mdot_t = @(m)well_model_stable_tubing(m,u,well,general_parameters);
    
%     h = 10;
%     mt = m0_t;
%     iter = 0;
%     [mdt wlp] = mdot_t(mt);
%     md = sum(abs(mdt));
%     while md > tol && wlp > 0 && iter < 1000
%         iter = iter+1;
%         mt = mt + h*mdt;
%         [mdt wlp] = mdot_t(mt);
%         md = sum(abs(mdt));
%         if h > 0.5
%             h = (1-5e-5)*h;
%         end
%     end
%     m_t = mt;
    
    options = optimset(...
        'Display','off',...
        'MaxFunEvals',500,...
        'MaxIter',100,...
        'TolFun', tol,...
        'TypicalX', m0_t...
    );
        
    m_t = fsolve(mdot_t,m0_t,options);

    
    % Searching for annulus solution using the bisection method    
    
    mdot_a = @(m)well_model_stable_annulus(...
        m,m_t,u,well,general_parameters);
    
%     figure(3);
%     plot(mdot_a(0:1:10000));
%     ylabel('$\dot{m}_{ga}$', 'interpreter','latex')    
%     xlabel('$m_{ga}$', 'interpreter','latex')

    a0 = 0;
    a2 = m0_a;
    while mdot_a(a2) > 0
        a0 = a2;
        a2 = 2*a2;
    end
    a1 = (a0 + a2)/2;
    
    ma = mdot_a(a1);
    while abs(ma) > tol
        if ma > 0
            a0 = a1;
            a1 = (a0 + a2)/2;
        else
            a2 = a1; 
            a1 = (a0 + a2)/2;
        end
        ma = mdot_a(a1);
    end
    m_a = a1;
    
    m = [m_a; m_t];
end

function mdot_t = well_model_stable_tubing(m,u,well,general_parameters)

% General parameters
R     = general_parameters(1);
g     = general_parameters(2);
T     = general_parameters(3);
p_m   = general_parameters(4);
p_r   = general_parameters(5);
M_g   = general_parameters(6);
rho_o = general_parameters(7);
rho_w = general_parameters(8);
A_t   = general_parameters(10);
C     = general_parameters(11);
C_pc  = general_parameters(12);

% Well specific parameters
V_t   = well(2);
L_w   = well(3);
r_wc  = well(4);
r_gor = well(5);
Q_max = well(6);

% States and inputs
m_gt = max(0,m(1));
m_lt = max(0,m(2));
m_t = m_gt + m_lt;
u_pc = u(1);
u_gl = u(2);

% Dependent parameters
T_t   = T;
r_glr = (1-r_wc)*r_gor;
rho_l = r_wc*rho_w + (1-r_wc)*rho_o;
nu_l  = 1/rho_l;

% Binder
p_p = R*T_t*m_gt/(M_g*V_t - M_g*nu_l*m_lt) - g*m_t/2/A_t;
p_ti = p_p + g*m_t/A_t;
a = r_glr*M_g*g*L_w/2/R/T_t;
p_bh = ((1+r_glr+a)*p_ti + rho_l*g*L_w)/(1+r_glr-a);

rho_p = rho_l*M_g*p_p*m_t/(rho_l*R*T_t*m_lt + M_g*p_p*m_gt);

w_p = C_pc*sqrt(rho_p*max(0,p_p-p_m))*u_pc;
w_lp = m_lt/m_t*w_p;
w_gp = w_p - w_lp;
w_lr = rho_l*Q_max*(1 - (1-C)*(p_bh/p_r) - C*(p_bh/p_r)^2);
w_gr = r_glr*w_lr;
w_gl = u_gl;

mdot_gt = w_gr + w_gl - w_gp;
mdot_lt = w_lr - w_lp;

% Return
mdot_t = [mdot_gt mdot_lt]';
end

function mdot_a = well_model_stable_annulus(...
    m_a,m_t,u,well,general_parameters)

% General parameters
R     = general_parameters(1);
g     = general_parameters(2);
T     = general_parameters(3);
M_g   = general_parameters(6);
rho_o = general_parameters(7);
rho_w = general_parameters(8);
A_a   = general_parameters(9);
A_t   = general_parameters(10);
C_iv  = general_parameters(13);

% Well specific parameters
V_a   = well(1);
V_t   = well(2);
r_wc  = well(4);

% States and inputs
m_ga = m_a;
m_gt = m_t(1);
m_lt = m_t(2);
m_t = m_gt + m_lt;
u_gl = u(2);

% Dependent parameters
T_t   = T;
T_a   = T;
rho_l = r_wc*rho_w + (1-r_wc)*rho_o;
nu_l  = 1/rho_l;

% Binder
p_p = R*T_t*m_gt/(M_g*V_t - M_g*nu_l*m_lt) - g*m_t/2/A_t;
p_ti = p_p + g*m_t/A_t;
p_ai = (R*T_a/V_a/M_g + g/2/A_a)*m_ga;

rho_gi = M_g/R/T_a*p_ai;

w_gi = C_iv*sqrt(rho_gi.*max(0,p_ai-p_ti));
w_gl = u_gl;

mdot_ga = w_gl - w_gi;

% Return
mdot_a = mdot_ga;

end


